(function(){
  var base62, ref$, keys, last, map, Str, isEqualToObject, presentationContext, pipeTransformation, compileTransformation, transpilation, compileAndExecuteSync, fromErrorValueTuple, processResponse, getJson, postJson, webClient;
  base62 = require('base62');
  ref$ = require('prelude-ls'), keys = ref$.keys, last = ref$.last, map = ref$.map, Str = ref$.Str;
  isEqualToObject = require('prelude-extension').isEqualToObject;
  presentationContext = require('./presentation-context');
  pipeTransformation = require('pipe-transformation'), compileTransformation = pipeTransformation.compileTransformation;
  transpilation = require('transpilation'), compileAndExecuteSync = transpilation.compileAndExecuteSync, fromErrorValueTuple = transpilation.fromErrorValueTuple;
  require('es6-promise').polyfill();
  require('isomorphic-fetch');
  processResponse = function(res){
    var ok;
    ok = res.ok;
    if (!ok) {
      return res.text().then(function(text){
        throw text;
      });
    } else {
      return res.json();
    }
  };
  getJson = function(url){
    return fetch(url).then(processResponse);
  };
  postJson = curry$(function(url, json){
    return fetch(url, {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json'
      },
      body: JSON.stringify(json)
    }).then(processResponse);
  });
  module.exports = webClient = function(config){
    var endPoint, compileDocument, compileQuery, compileLatestQuery, compilePresentationSync, compilePresentation, execute, generateUid, getAllTags, loadDefaultDocument, loadQuery, loadLatestQuery, requireDeps, saveDocument, ref$;
    endPoint = config.endPoint;
    compileDocument = function(){
      var cache;
      cache = {};
      return function(document){
        var queryId, dataSourceCue, query, clientExternalLibs, transpilation, transformation, presentation;
        if (document != null) {
          queryId = document.queryId, dataSourceCue = document.dataSourceCue, query = document.query, clientExternalLibs = document.clientExternalLibs, transpilation = document.transpilation, transformation = document.transformation, presentation = document.presentation;
        }
        if (!!cache[queryId]) {
          return Promise.resolve(cache[queryId]);
        }
        return requireDeps(clientExternalLibs != null
          ? clientExternalLibs
          : []).then(function(){
          return compileTransformation(transformation, transpilation.transformation).then(function(transformationFunction){
            return compilePresentation(presentation, transpilation.presentation).then(function(presentationFunction){
              return cache[queryId] = {
                document: document,
                execute: curry$(function(cache, compiledParameters){
                  var opInfo;
                  opInfo = {
                    document: document
                  };
                  return execute(dataSourceCue, query, transpilation.query, compiledParameters, cache, generateUid(), opInfo).then(function(arg$){
                    var result;
                    if (arg$ != null) {
                      result = arg$.result;
                    }
                    return result;
                  });
                }),
                transformationFunction: transformationFunction,
                presentationFunction: presentationFunction
              };
            });
          });
        });
      };
    }();
    compileQuery = function(){
      var cache;
      cache = {};
      return function(queryId){
        return (!!cache[queryId]
          ? Promise.resolve(cache[queryId])
          : loadQuery(queryId)).then(function(document){
          return compileDocument(cache[queryId] = document);
        });
      };
    }();
    compileLatestQuery = function(){
      var cache;
      cache = {};
      return function(branchId){
        return (!!cache[branchId]
          ? Promise.resolve(cache[branchId])
          : loadLatestQuery(branchId)).then(function(document){
          return compileDocument(cache[branchId] = document);
        });
      };
    }();
    compilePresentationSync = function(presentation, language){
      return compileAndExecuteSync(presentation, language, presentationContext());
    };
    compilePresentation = fromErrorValueTuple(compilePresentationSync);
    execute = function(){
      var previousCall;
      previousCall = null;
      return function(dataSourceCue, query, transpilationLanguage, compiledParameters, cache, opId, opInfo){
        var args;
        args = {
          dataSourceCue: dataSourceCue,
          query: query,
          transpilationLanguage: transpilationLanguage,
          compiledParameters: compiledParameters
        };
        if (!!cache && !!previousCall && isEqualToObject(previousCall.args, args)) {
          return new Promise(function(res, rej){
            var ref$, result, executionEndTime;
            ref$ = previousCall.resultWithMetadata, result = ref$.result, executionEndTime = ref$.executionEndTime;
            return res({
              fromCache: true,
              executionDuration: 0,
              executionEndTime: executionEndTime,
              result: result
            });
          });
        } else {
          return postJson(endPoint + "/apis/execute", {
            dataSourceCue: dataSourceCue,
            query: query,
            transpilationLanguage: transpilationLanguage,
            compiledParameters: compiledParameters,
            cache: cache,
            opId: opId,
            opInfo: opInfo
          }).then(function(resultWithMetadata){
            previousCall = {
              args: args,
              resultWithMetadata: resultWithMetadata
            };
            return resultWithMetadata;
          });
        }
      };
    }();
    generateUid = function(){
      return base62.encode(Date.now());
    };
    getAllTags = function(){
      return getJson(endPoint + "/apis/tags");
    };
    loadDefaultDocument = function(dataSourceCue, transpilationLanguage){
      return postJson(endPoint + "/apis/defaultDocument", [dataSourceCue, transpilationLanguage]);
    };
    loadQuery = function(queryId){
      return getJson(endPoint + "/apis/queries/" + queryId);
    };
    loadLatestQuery = function(branchId){
      return getJson(endPoint + "/apis/branches/" + branchId).then(function(it){
        return loadQuery(it[0].latestQuery.queryId);
      });
    };
    requireDeps = function(clientExternalLibs){
      if (clientExternalLibs.length === 0) {
        return Promise.resolve(clientExternalLibs);
      }
      return Promise.all(map(function(url){
        return new Promise(function(res){
          var element, script, link, this$ = this;
          element = (function(){
            var x$, y$;
            switch (last(url.split('.'))) {
            case 'js':
              x$ = script = document.createElement('script');
              x$.src = url;
              return x$;
              break;
            case 'css':
              y$ = link = document.createElement('link');
              y$.type = 'text/css';
              y$.rel = 'stylesheet';
              y$.href = url;
              return y$;
            }
          }());
          element.onload = function(){
            return res(url);
          };
          return document.head.appendChild(element);
        });
      })(
      clientExternalLibs));
    };
    saveDocument = postJson(endPoint + "/apis/save");
    return ref$ = import$(import$({}, transpilation), pipeTransformation), ref$.compilePresentation = compilePresentation, ref$.compilePresentationSync = compilePresentationSync, ref$.compileDocument = compileDocument, ref$.compileQuery = compileQuery, ref$.compileLatestQuery = compileLatestQuery, ref$.execute = execute, ref$.getAllTags = getAllTags, ref$.generateUid = generateUid, ref$.loadDefaultDocument = loadDefaultDocument, ref$.loadLatestQuery = loadLatestQuery, ref$.loadQuery = loadQuery, ref$.requireDeps = requireDeps, ref$.saveDocument = saveDocument, ref$;
  };
  function curry$(f, bound){
    var context,
    _curry = function(args) {
      return f.length > 1 ? function(){
        var params = args ? args.concat() : [];
        context = bound ? context || this : this;
        return params.push.apply(params, arguments) <
            f.length && arguments.length ?
          _curry.call(context, params) : f.apply(context, params);
      } : f;
    };
    return _curry();
  }
  function import$(obj, src){
    var own = {}.hasOwnProperty;
    for (var key in src) if (own.call(src, key)) obj[key] = src[key];
    return obj;
  }
}).call(this);
