(function(){
  var ref$, map, id, filter, Obj, pairsToObj;
  ref$ = require('prelude-ls'), map = ref$.map, id = ref$.id, filter = ref$.filter, Obj = ref$.Obj, pairsToObj = ref$.pairsToObj;
  module.exports = function(arg$){
    var Plottable, d3, plotChart, nv;
    Plottable = arg$.Plottable, d3 = arg$.d3, plotChart = arg$.plotChart, nv = arg$.nv;
    return new Plottable(function(view, result, arg$, continuation){
      var traits, category, onselected, ondeselected, width, padding, n, size, x, y, xAxis, yAxis, color, domainByTrait, cross, plot, svg, cell, brushCell, brushstart, brushmove, brushend, brush;
      traits = arg$.traits, category = arg$.category, onselected = arg$.onselected, ondeselected = arg$.ondeselected;
      if (!traits) {
        traits = Obj.keys(result[0]);
      }
      if (!category) {
        category = function(){
          return "any-category";
        };
      }
      width = view.clientWidth;
      padding = 10;
      n = traits.length;
      size = (width - n * padding) / n;
      x = d3.scale.linear().range([padding / 2, size - padding / 2]);
      y = d3.scale.linear().range([size - padding / 2, padding / 2]);
      xAxis = d3.svg.axis().scale(x).orient('bottom').ticks(5).tickSize(size * n);
      yAxis = d3.svg.axis().scale(y).orient('left').ticks(5).tickSize(-1 * size * n);
      color = d3.scale.category10();
      domainByTrait = compose$(map(function(t){
        return [
          t, d3.extent(result, function(d){
            return d[t];
          })
        ];
      }), pairsToObj)(
      traits);
      cross = curry$(function(a, b){
        var i$, ref$, len$, i, j$, ref1$, len1$, j, results$ = [];
        for (i$ = 0, len$ = (ref$ = (fn$())).length; i$ < len$; ++i$) {
          i = ref$[i$];
          for (j$ = 0, len1$ = (ref1$ = (fn1$())).length; j$ < len1$; ++j$) {
            j = ref1$[j$];
            results$.push({
              i: i,
              j: j,
              x: a[i],
              y: b[j]
            });
          }
        }
        return results$;
        function fn$(){
          var i$, to$, results$ = [];
          for (i$ = 0, to$ = a.length - 1; i$ <= to$; ++i$) {
            results$.push(i$);
          }
          return results$;
        }
        function fn1$(){
          var i$, to$, results$ = [];
          for (i$ = 0, to$ = b.length - 1; i$ <= to$; ++i$) {
            results$.push(i$);
          }
          return results$;
        }
      });
      plot = function(p){
        var cell;
        cell = d3.select(this);
        x.domain(domainByTrait[p.x]);
        y.domain(domainByTrait[p.y]);
        cell.append('rect').attr('class', 'frame').attr('x', padding / 2).attr('y', padding / 2).attr('width', size - padding).attr('height', size - padding);
        return cell.selectAll('circle').data(result).enter().append('circle').attr("cx", function(d){
          return x(d[p.x]);
        }).attr("cy", function(d){
          return y(d[p.y]);
        }).attr("r", 3).style("fill", function(d){
          return color(category(d));
        });
      };
      svg = d3.select(view).append('svg').attr('class', 'correlation-matrix').attr('width', size * n + padding).attr('height', size * n + padding).append('g').attr('transform', "translate(" + padding + "," + padding / 2 + ")");
      svg.selectAll('.x.axis').data(traits).enter().append('g').attr('class', 'x axis').attr('transform', function(d, i){
        return "translate(" + (n - i - 1) * size + ",0)";
      }).each(function(d){
        x.domain(domainByTrait[d]);
        return d3.select(this).call(xAxis);
      });
      svg.selectAll('.y.axis').data(traits).enter().append('g').attr('class', 'y axis').attr('transform', function(d, i){
        return "translate(0," + i * size + ")";
      }).each(function(d){
        y.domain(domainByTrait[d]);
        return d3.select(this).call(yAxis);
      });
      cell = svg.selectAll('.cell').data(cross(traits, traits)).enter().append('g').attr("class", "cell").attr("transform", function(d){
        return "translate(" + (n - d.i - 1) * size + "," + d.j * size + ")";
      }).each(plot);
      cell.filter(function(d){
        return d.i === d.j;
      }).append('text').attr('x', padding).attr('y', padding).attr('dy', '.71em').text(function(it){
        return it.x;
      });
      brushCell = null;
      brushstart = function(p){
        var brushCell;
        if (!deepEq$(brushCell, this, '===')) {
          d3.select(brushCell).call(brush.clear());
          x.domain(domainByTrait[p.x]);
          y.domain(domainByTrait[p.y]);
          return brushCell = this;
        }
      };
      brushmove = function(p){
        var e, x$;
        e = brush.extent();
        x$ = svg.selectAll('circle');
        x$.classed('hidden', function(d){
          return e[0][0] > d[p.x] || d[p.x] > e[1][0] || e[0][1] > d[p.y] || d[p.y] > e[1][1];
        });
        return x$;
      };
      brushend = function(){
        if (brush.empty()) {
          svg.selectAll('.hidden').classed('hidden', false);
          if (!!ondeselected) {
            return svg.select(".cell").selectAll("circle:not(.hidden)").call(function(d){
              return ondeselected(d.data(), d);
            });
          }
        } else {
          if (!!onselected) {
            return svg.select(".cell").selectAll("circle:not(.hidden)").call(function(d){
              return onselected(d.data(), d);
            });
          }
        }
      };
      brush = d3.svg.brush().x(x).y(y).on('brushstart', brushstart).on('brush', brushmove).on('brushend', brushend);
      cell.call(brush);
    });
  };
  function compose$() {
    var functions = arguments;
    return function() {
      var i, result;
      result = functions[0].apply(this, arguments);
      for (i = 1; i < functions.length; ++i) {
        result = functions[i](result);
      }
      return result;
    };
  }
  function curry$(f, bound){
    var context,
    _curry = function(args) {
      return f.length > 1 ? function(){
        var params = args ? args.concat() : [];
        context = bound ? context || this : this;
        return params.push.apply(params, arguments) <
            f.length && arguments.length ?
          _curry.call(context, params) : f.apply(context, params);
      } : f;
    };
    return _curry();
  }
  function deepEq$(x, y, type){
    var toString = {}.toString, hasOwnProperty = {}.hasOwnProperty,
        has = function (obj, key) { return hasOwnProperty.call(obj, key); };
    var first = true;
    return eq(x, y, []);
    function eq(a, b, stack) {
      var className, length, size, result, alength, blength, r, key, ref, sizeB;
      if (a == null || b == null) { return a === b; }
      if (a.__placeholder__ || b.__placeholder__) { return true; }
      if (a === b) { return a !== 0 || 1 / a == 1 / b; }
      className = toString.call(a);
      if (toString.call(b) != className) { return false; }
      switch (className) {
        case '[object String]': return a == String(b);
        case '[object Number]':
          return a != +a ? b != +b : (a == 0 ? 1 / a == 1 / b : a == +b);
        case '[object Date]':
        case '[object Boolean]':
          return +a == +b;
        case '[object RegExp]':
          return a.source == b.source &&
                 a.global == b.global &&
                 a.multiline == b.multiline &&
                 a.ignoreCase == b.ignoreCase;
      }
      if (typeof a != 'object' || typeof b != 'object') { return false; }
      length = stack.length;
      while (length--) { if (stack[length] == a) { return true; } }
      stack.push(a);
      size = 0;
      result = true;
      if (className == '[object Array]') {
        alength = a.length;
        blength = b.length;
        if (first) {
          switch (type) {
          case '===': result = alength === blength; break;
          case '<==': result = alength <= blength; break;
          case '<<=': result = alength < blength; break;
          }
          size = alength;
          first = false;
        } else {
          result = alength === blength;
          size = alength;
        }
        if (result) {
          while (size--) {
            if (!(result = size in a == size in b && eq(a[size], b[size], stack))){ break; }
          }
        }
      } else {
        if ('constructor' in a != 'constructor' in b || a.constructor != b.constructor) {
          return false;
        }
        for (key in a) {
          if (has(a, key)) {
            size++;
            if (!(result = has(b, key) && eq(a[key], b[key], stack))) { break; }
          }
        }
        if (result) {
          sizeB = 0;
          for (key in b) {
            if (has(b, key)) { ++sizeB; }
          }
          if (first) {
            if (type === '<<=') {
              result = size < sizeB;
            } else if (type === '<==') {
              result = size <= sizeB
            } else {
              result = size === sizeB;
            }
          } else {
            first = false;
            result = size === sizeB;
          }
        }
      }
      stack.pop();
      return result;
    }
  }
}).call(this);
