(function(){
  var ref$, map, minimum, maximum, pow;
  ref$ = require('prelude-ls'), map = ref$.map, minimum = ref$.minimum, maximum = ref$.maximum, pow = ref$.pow;
  module.exports = function(arg$){
    var Plottable, d3;
    Plottable = arg$.Plottable, d3 = arg$.d3;
    return new Plottable(function(view, result, options, continuation){
      var margin, x, y, size, yRange, yAxis, xAxis, tooltip, leastSquares, width, height, x$, xScale, y$, yScale, z$, sizeScale, svg, tip, circle, leastSquaresCoeff, line, x1, y1, x2, y2, trendline, z1$;
      margin = options.margin, x = options.x, y = options.y, size = options.size, yRange = options.yRange, yAxis = options.yAxis, xAxis = options.xAxis, tooltip = options.tooltip;
      leastSquares = function(xSeries, ySeries){
        var reduceSumFunc, xBar, yBar, ssXX, ssYY, ssXY, slope, intercept, rSquare;
        reduceSumFunc = function(prev, cur){
          return prev + cur;
        };
        xBar = xSeries.reduce(reduceSumFunc) * 1.0 / xSeries.length;
        yBar = ySeries.reduce(reduceSumFunc) * 1.0 / ySeries.length;
        ssXX = xSeries.map(function(d){
          return Math.pow(d - xBar, 2);
        }).reduce(reduceSumFunc);
        ssYY = ySeries.map(function(d){
          return Math.pow(d - yBar, 2);
        }).reduce(reduceSumFunc);
        ssXY = xSeries.map(function(d, i){
          return (d - xBar) * (ySeries[i] - yBar);
        }).reduce(reduceSumFunc);
        slope = ssXY / ssXX;
        intercept = yBar - xBar * slope;
        rSquare = Math.pow(ssXY, 2) / (ssXX * ssYY);
        return [slope, intercept, rSquare];
      };
      yRange = import$({
        min: compose$(map(y), minimum),
        max: compose$(map(y), maximum)
      }, yRange);
      width = view.clientWidth - margin.left - margin.right;
      height = view.clientHeight - margin.top - margin.bottom;
      x$ = xScale = d3.scale.linear().range([0, width]);
      x$.domain([
        compose$(map(x), minimum)(
        result), compose$(map(x), maximum)(
        result)
      ]);
      y$ = yScale = d3.scale.linear().range([height, 0]);
      y$.domain([yRange.min(result), yRange.max(result)]);
      z$ = sizeScale = d3.scale.linear().range([3, 10]);
      z$.domain([
        minimum(
        map(size)(
        result)), maximum(
        map(size)(
        result))
      ]);
      svg = d3.select(view).append('div').attr('style', "position: absolute; left: 0px; top: 0px; width: 100%; height: 100%").append('svg').attr('class', 'regression').attr('width', width + margin.left + margin.right).attr('height', height + margin.top + margin.bottom).append('g').attr('transform', "translate(" + margin.left + "," + margin.top + ")");
      if (tooltip !== null) {
        tip = d3.tip().attr('class', 'd3-tip').offset([-10, 0]).html(tooltip);
        svg.call(tip);
      }
      svg.append('g').attr('class', 'x axis');
      svg.append('g').attr('class', 'y axis').attr('transform', "translate(0, 0)").call(function(){
        return d3.svg.axis().scale(yScale).orient('left').tickFormat(yAxis.format);
      }());
      svg.append('g').attr('class', 'x axis').attr('transform', "translate(0, " + (view.clientHeight - margin.bottom - margin.top) + ")").call(function(){
        return d3.svg.axis().scale(xScale).orient('bottom').tickFormat(xAxis.format);
      }());
      circle = svg.selectAll('circle').data(result).enter().append('circle').attr("cx", compose$(x, xScale)).attr("cy", compose$(y, yScale)).attr("r", compose$(size, sizeScale)).style("fill", function(d){
        return "blue";
      });
      if (tooltip !== null) {
        circle.on('mouseover', tip.show).on('mouseout', tip.hide);
      }
      leastSquaresCoeff = leastSquares(map(x, result), map(y, result));
      line = function(it){
        return leastSquaresCoeff[0] * it + leastSquaresCoeff[1];
      };
      x1 = compose$(map(x), minimum)(
      result);
      y1 = line(x1);
      x2 = compose$(map(x), maximum)(
      result);
      y2 = line(x2);
      trendline = svg.selectAll('.trendline').data([[x1, y1, x2, y2]]).enter().append('line').attr('class', 'trendline').attr('x1', function(d){
        return xScale(d[0]);
      }).attr('y1', function(d){
        return yScale(d[1]);
      }).attr('x2', function(d){
        return xScale(d[2]);
      }).attr('y2', function(d){
        return yScale(d[3]);
      }).attr('stroke', 'black').attr('stroke-width', 1);
      z1$ = svg;
      z1$.append('text').text("y = " + d3.format('0.2f')(leastSquaresCoeff[0]) + " x + " + d3.format('0.2f')(leastSquaresCoeff[1])).attr('class', 'text-label').attr('x', function(d){
        return margin.left * 0.6;
      }).attr('y', function(d){
        if (yScale(y1 > view.clientHeight / 2)) {
          return margin.bottom * 0.6;
        } else {
          return view.clientHeight - 2.4 * margin.bottom;
        }
      });
      z1$.append('text').text("r square = " + d3.format('0.2f')(leastSquaresCoeff[2])).attr('class', 'text-label').attr('x', function(d){
        return margin.left * 0.6;
      }).attr('y', function(d){
        if (yScale(y1 > view.clientHeight / 2)) {
          return margin.bottom;
        } else {
          return view.clientHeight - 2 * margin.bottom;
        }
      });
    }, {
      x: function(it){
        return it.x;
      },
      y: function(it){
        return it.y;
      },
      size: function(it){
        return it.size;
      },
      yAxis: {
        format: d3.format('0.2f'),
        label: 'Y'
      },
      xAxis: {
        format: d3.format('0.2f'),
        label: 'X'
      },
      yRange: null,
      tooltip: null,
      margin: {
        top: 20,
        right: 20,
        bottom: 50,
        left: 50
      }
    });
  };
  function import$(obj, src){
    var own = {}.hasOwnProperty;
    for (var key in src) if (own.call(src, key)) obj[key] = src[key];
    return obj;
  }
  function compose$() {
    var functions = arguments;
    return function() {
      var i, result;
      result = functions[0].apply(this, arguments);
      for (i = 1; i < functions.length; ++i) {
        result = functions[i](result);
      }
      return result;
    };
  }
}).call(this);
